/***************************************************************************
 *   Copyright (C) 2005 by Christoph Hormann <chris_hormann@gmx.de>        *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#define KIO_POV_NAME "kio-pov"
#define KIO_POV_VERSION "0.2"

#include <kstandarddirs.h>
#include <kprocess.h>

#include <qstring.h>
#include <qcstring.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qdatastream.h>
#include <qregexp.h>

#include "povscene.h"

/* --------------------------------------------------------------------------- */

POVScene::POVScene(const QString &scene)
{
	level = info_start;
	cache_status = cache_unknown;
	lines = 0;
	pov_version = 0.0;
	pov_exe = "povray";
	RenderSz_X = -1;
	RenderSz_Y = -1;
	Anim_Framerate = 30;
	Anim_Bitrate = 300;

	// find out the tmp dir
	char *s;
	QString TmpDir;
	if ((s = getenv("TMPDIR"))) TmpDir = QString(s);
	else TmpDir = QString("/tmp");
	
	// assemble names of other files
	pov_name = scene;
	QFileInfo finfo_pov(pov_name);
	QString dir = finfo_pov.dirPath(true);
	scene_dir = dir;

	fname = finfo_pov.fileName();
	img_name = finfo_pov.fileName();
	dir.prepend(TmpDir + "/" + KIO_POV_NAME);
	dir.append("/");
	img_name.prepend(dir);
	ini_name = img_name + ".ini";
	log_name = img_name + ".log";
	thumb_name = img_name + "_small.png";
	anim_name = img_name + ".mpeg";
	img_name_base = img_name;
	img_name.append(".png");

	render_dir = dir;

}

/* --------------------------------------------------------------------------- */

bool POVScene::Prepare(void)
{
	if (!KStandardDirs::exists(render_dir))
	{
		KStandardDirs::makeDir(render_dir);
		return false;
	}
	
	return true;
}

/* --------------------------------------------------------------------------- */

int POVScene::cached(void)
{
	if (cache_status == cache_unknown)
	{
		// if ini options indicate animation look for mpeg file
		if (isAnimation())
		{
			if (KStandardDirs::exists(pov_name) && KStandardDirs::exists(anim_name))
			{
				QFileInfo finfo_pov(pov_name);
				QFileInfo finfo_mpeg(anim_name);
				if (finfo_pov.lastModified() > finfo_mpeg.lastModified())
					cache_status = cache_old_anim;
				else
					cache_status = cache_anim;
			}
		}

		// if no animation or no mpeg found check still
		if (cache_status == cache_unknown)
		{
			if (KStandardDirs::exists(pov_name) && KStandardDirs::exists(img_name))
			{
				QFileInfo finfo_pov(pov_name);
				QFileInfo finfo_png(img_name);
				if (finfo_pov.lastModified() > finfo_png.lastModified())
					cache_status = cache_old;
				else
					cache_status = cache_valid;
			}
		}

		// no image/video but log -> error
		if (cache_status == cache_unknown)
		{
			if (KStandardDirs::exists(log_name))
			{
				cache_status = cache_error;
			}
			else cache_status = cache_none;
		}
	}

	return cache_status;
}

/* --------------------------------------------------------------------------- */

bool POVScene::Analyze(int mode)
{
	// if scene is already analyzed to the required level exit.
	if (level >= mode) return false;
	
	if (KStandardDirs::exists(pov_name))
	{
		QFile pov_file( pov_name );
		if ( pov_file.open( IO_ReadOnly ) )
		{
			QTextStream pov_stream( &pov_file );
						
			QString pov_line;
			int Found = 0;
	
			if (mode==info_basic) // version and options only
			{
				for (int i=0;i<POV_SCAN_LINES;i++)
				{
					pov_line = pov_stream.readLine();
	
					if (pov_line.left(8) == "#version")
					{
						if (pov_line.find("unofficial megapov", 0)>=0)
						{
							pov_exe = "megapov";
						}
						Found++;
					}
					else if (pov_line.find( QRegExp("^//[ ]+[-+][^ -]"), 0 )==0)
					{
						scene_options = pov_line.mid(3);
						Found++;
					}
					if (Found>1) break;
				}
				level = info_basic;
			}
			else
			{
				int i=0;
				int pos;
				
				while (!pov_stream.atEnd())
				{
					pov_line = pov_stream.readLine();
	
					if (i<POV_SCAN_LINES)
					{
						if (pov_line.left(8) == "#version")
						{
							if ((pos = pov_line.find("unofficial megapov", 0))>=0)
							{
								pov_exe = "megapov";
								int pos2 = pov_line.find(";", pos);
								pov_version = pov_line.mid(pos+18, pos2-pos-18).toFloat();
							}
							else
							{
								pos = pov_line.find(";", 8);
								pov_version = pov_line.mid(8, pos-8).toFloat();
							}
						}
						else if (pov_line.find( QRegExp("^//[ ]+[-+][^ -]"), 0 )==0)
						{
							scene_options = pov_line.mid(3);
						}
						else if (pov_line.find( QRegExp("^// [ ]*Author:"), 0 )==0)
						{
							scene_author = pov_line.mid(pov_line.find( "Author:", 0 )+7);
							if (scene_author[0]==' ') scene_author.remove(0,1);
						}
						else if (pov_line.find( QRegExp("^// [ ]*Description:"), 0 )==0)
						{
							scene_description = pov_line.mid(pov_line.find( "Description:", 0 )+12);
							if (scene_description[0]==' ') scene_description.remove(0,1);
						}
					}
					if (pov_line.left(8) == "#include")
					{
						include_list += pov_line.section( '"', 1, 1 );
					}
					i++;
				}
				lines = i;
				level = info_detailed;
			}
			pov_file.close();
			
			return false;
		}
	}
	
	return true;
}

/* --------------------------------------------------------------------------- */

int POVScene::AnalyzeLog(void)
{
	if (RenderSz_X >= 0) return 2; 
	
	// first read log file
	QString log_text;
	QFile log_fl( log_name );
	if ( log_fl.open( IO_ReadOnly ) )
	{
		QTextStream log_stream( &log_fl );
		log_text = log_stream.read();
		log_text.replace('\r', '\n');
		log_fl.close();
	}
	else
	{
		RenderSz_X = 0;
		RenderSz_Y = 0;
		return 1;
	}
	
	int pos, pos2, pos3;

	pos = log_text.findRev( QRegExp("[\\r\\n]Image Resolution") );
	if (pos >= 0)
	{
		pos2 = log_text.find( " x ", pos );
		RenderSz_X = log_text.mid(pos+18,pos2-pos-18).toULong();
		pos3 = log_text.find( QRegExp("[\\r\\n]"), pos2 );
		RenderSz_Y = log_text.mid(pos2+2,pos3-pos2-2).toULong();
		return 0;
	}
	else // no image dimensions found
	{
		RenderSz_X = 0;
		RenderSz_Y = 0;
		return 1;
	}
}

/* --------------------------------------------------------------------------- */

QString POVScene::ini_options(void)
{
	// current options known - do not read
	if (options_string!=QString::null)
	{
		return options_string;
	}
	else if (KStandardDirs::exists(ini_name))
	{
		QFile ini_file( ini_name );
		if ( ini_file.open( IO_ReadOnly ) )
		{
			QTextStream ini_stream( &ini_file );

			QString ini_text = ini_stream.read();
			ini_file.close();

			int pos = ini_text.find( QRegExp("[\\r\\n]; @@ ") );
			pos += 6;
			int end_pos = ini_text.find( QRegExp("[\\r\\n]"), pos );
			if ( pos >= 6  && end_pos > pos)
			{
				options_string = ini_text.mid(pos,end_pos-pos);
				return options_string;
			}
		}
	}

	return QString::null;
}

/* --------------------------------------------------------------------------- */

bool POVScene::isAnimation(void)
{
	QString Options=ini_options();

	if (Options!=QString::null)
	{
		if (Options.find(QString("final_frame"), 0, false) > -1) return true;
		if (Options.find(QString("+kff"), 0, false) > -1) return true;
	}
	return false;
}

/* --------------------------------------------------------------------------- */

bool POVScene::write_options(const QString Options)
{
	QFile ini_file( ini_name );
	if ( ini_file.open( IO_WriteOnly ) )
	{
		QTextStream ini_stream( &ini_file );

		ini_stream << "; POV-Ray ini file generated by ";
		ini_stream << KIO_POV_NAME << " version " << KIO_POV_VERSION << endl;

		if (Options!=QString::null)
		{
			QString Options_Line = Options;
			Options_Line.replace( QRegExp("[\r\n]"), " " );
			ini_stream << "\n; @@ " << Options_Line << endl;
			ini_stream << Options << endl;

			options_string = Options_Line;
		}
		else options_string = "";

		// always add Library_Path for the scene dir since we render from the cache dir
		ini_stream << QString("\nLibrary_Path=\"" + scene_dir + "\"\n");

		// always add the input file at the end
		ini_stream << QString("Input_File_Name=\"" + pov_name + "\"\n");
		ini_file.close();

		return true;
	}
	else return false;
}

/* --------------------------------------------------------------------------- */
