/***************************************************************************
 *   Copyright (C) 2005 by Christoph Hormann <chris_hormann@gmx.de>        *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef _povscene_H_
#define _povscene_H_

#define POV_SCAN_LINES 50

#include <qstring.h>

/**
	This class handles a POV-Ray scene and
	the files in the tmp directory used to cache render results
 */
class POVScene
{

	public:
		POVScene(const QString &scene);
		~POVScene() {;}

		enum { info_start = 0, info_basic = 1, info_detailed = 2 };
		enum { cache_unknown = 0, cache_none = 1, cache_old = 2, cache_old_anim = 3, cache_error = 4, cache_valid = 5, cache_anim = 6 };

		/// scene file name without path
		inline QString filename(void) const { return(fname); }
		/// complete scene file path
		inline QString pov_file(void) const { return(pov_name); }
		/// complete image file path (in cache dir)
		inline QString img_file(void) const { return(img_name); }
		/// complete animation file path (in cache dir)
		inline QString anim_file(void) const { return(anim_name); }
		/// image file base (without extension) (in cache dir)
		inline QString img_file_base(void) const { return(img_name_base); }
		/// complete thumbnail image file path (in cache dir)
		inline QString thumbnail_file(void) const { return(thumb_name); }
		/// complete ini file path (in cache dir)
		inline QString ini_file(void) const { return(ini_name); }
		/// complete log file path (in cache dir)
		inline QString log_file(void) const { return(log_name); }
		/// render directory (in cache dir)
		inline QString render_directory(void) const { return(render_dir); }
		/// scene directory
		inline QString scene_directory(void) const { return(scene_dir); }
		/**
			the POV-Ray executable name to use
			determined from #version directive in the scene file
		*/
		inline QString povray_executable(void) { Analyze(info_basic); return(pov_exe); }
		/// suggested render options from the scene file header comment
		inline QString options(void) { Analyze(info_basic); return(scene_options); }
		/// scene author from the scene file header comment
		inline QString author(void) { Analyze(info_detailed); return(scene_author); }
		/// scene description from the scene file header comment
		inline QString description(void) { Analyze(info_detailed); return(scene_description); }
		/// list of files #included in the scene
		inline QStringList includes(void) { Analyze(info_detailed); return(include_list); }
		/// scene file line count
		inline unsigned int line_count(void) { Analyze(info_detailed); return(lines); }
		/// width of cached render determined from log
		inline long render_width(void) { AnalyzeLog(); return(RenderSz_X); }
		/// height of cached render determined from log
		inline long render_height(void) { AnalyzeLog(); return(RenderSz_Y); }
		/// frame rate for animation
		inline int framerate(void) { return(Anim_Framerate); }
		/// bit rate for animation
		inline int bitrate(void) { return(Anim_Bitrate); }
		/**
			the POV-Ray version the scene is designed for
			determined from #version directive in the scene file
			defaults to 0.0 if no #version is found.
		 */
		inline float povray_version(void) { Analyze(info_detailed); return(pov_version); }

		/// prepare for render
		bool Prepare(void);
		/// determine if there is a valid cached render
		int cached(void);
		/// read the options used for the last (cached) render
		QString ini_options(void);
		/**
			write options to the ini file in the cache dir
			to use them in the next render

			@param Options QString containing the options to write

			@returns
				true if writing was successful
				false otherwise
		*/
		bool write_options(const QString Options);

		/**
			checks if the scene with the current options will be rendered as an animation

			@returns
				true if it is an animation
				false otherwise
		 */
		bool isAnimation(void);

		/**
			sets the frame- and bitrate for assembling animation

			@param Framerate frame rate to use
			@param Bitrate bit rate to use

		 */
		inline void setAnimationParameters(int Framerate, int Bitrate) { Anim_Framerate = Framerate; Anim_Bitrate = Bitrate; };

	private:
		bool Analyze(int mode);
		int AnalyzeLog(void);
		QString fname;
		QString pov_name;
		QString img_name;
		QString img_name_base;
		QString anim_name;
		QString thumb_name;
		QString ini_name;
		QString log_name;
		QString render_dir;
		QString scene_dir;
		QString pov_exe;
		QString scene_options;
		QString scene_author;
		QString scene_description;
		QString options_string;
		QStringList include_list;
		unsigned int lines;
		float pov_version;
		int level;
		int cache_status;
		long RenderSz_X;
		long RenderSz_Y;
		int Anim_Framerate;
		int Anim_Bitrate;
};

#endif
